
class Batch():
    @staticmethod
    def download(obj_list):
        for obj in obj_list:
            obj.download()

    @staticmethod
    def upload(obj_list, to=None):
        for obj in obj_list:
            if to is not None:
                obj._parent = to
            obj.upload()

    @staticmethod
    def upload_meta(obj_list, to=None):
        for obj in obj_list:
            if to is not None:
                obj._parent = to
            obj.upload_meta()

    @staticmethod
    def get(obj_list):
        for obj in obj_list:
            obj.get()

    @staticmethod
    def update(obj_list):
        for obj in obj_list:
            obj.update()



import rinocloud


class Query():

    OPERATORS = [
        'eq', 'lt', 'lte', 'gt', 'gte', 'ne', 'in', 'nin', 'exists', 'or', 'string_contains'
    ]

    def __init__(self, query_dict={}, results='The query method has not yet been called.'):
        self.query_dict = query_dict
        self.results = results

    def __repr__(self):
        return "<rinocloud.Query>"

    @classmethod
    def extract_filter_operator(cls, parameter):
        for op in cls.OPERATORS:
            underscored = '__%s' % op
            if parameter.endswith(underscored):
                return parameter[:-len(underscored)], op
        return parameter, None

    def filter(self, **kw):
        for name, value in kw.items():
            attr, operator = Query.extract_filter_operator(name)
            if operator is None:
                self.query_dict[attr] = value
            elif operator is 'or':
                option_list = []
                if '$or' in self.query_dict:
                    option_list = self.query_dict['$or']
                    option_list.append({attr: value})
                    self.query_dict['$' + operator] = option_list
                else:
                    option_list.append({attr: value})
                    self.query_dict['$' + operator] = option_list
            else:
                if attr in self.query_dict:
                    self.query_dict[attr]['$' + operator] = value
                else:
                    self.query_dict[attr] = {'$' + operator: value}
        return self

    def query(self, truncate_metadata=True, limit=20, offset=0, **kw):
        r = rinocloud.http.query(self.query_dict, truncate_metadata, limit, offset)
        assert r.status_code == 200, "Query failed: %s" % r.text
        reply = r.json()["result"]
        return [rinocloud.Object()._process_response_metadata(item, **kw) for item in reply]

    def count(self, **kw):
        r = rinocloud.http.count(self.query_dict)
        assert r.status_code == 200, "Query failed: %s" % r.text
        return r.json()["count"]



import json
import os
import rinocloud


class Object():

    def __init__(self, **kw):
        """
        Initialise a bunch of variables
        """
        # these are some rinocloud variables we dont mind showing to the users
        self.id = None
        self.created_on = None
        self.updated_on = None
        self._cloud_name = None
        self.name = None

        # these are some variables we will keep hidden, marked with underscore
        self._size = None
        self._parent = None

        # this needs to be set by the user in order to save locally
        # they just call self.set_local_path
        self._path = rinocloud.path
        self.filepath = ''

        # lets set all the passed kwargs to this object
        for key, value in kw.iteritems():
            setattr(self, key, value)

    def __repr__(self):
        return "<rinocloud.Object name=%s id=%s>" % (self.name, self.id)

    def increment_name(self, name, i):
        """
        takes something like
            test.txt
        and returns
            test1.txt
        """
        if i == 0:
            return name
        split = name.split('.')
        split[-2] = split[-2] + str(i)
        return '.'.join(split)

    def set_name(self, name, overwrite=False, increment=True, path=None, create_dir=False):
        """
        Sets the name of the file to be saved.

        @params
            name - the name to the file
            increment - whether or not to increment the filename if there is an existing file ie test.txt => test1.txt
            overwrite - whether or not to overwrite existing local file, renders increment redundant
            path - folder path to save the file too
            create_dir - whether to create the directory if it doesnt exist.
        """

        self._path = rinocloud.path
        # if path is not None:
        #   self.set_local_path(path, create_dir)

        # check if the file exists
        exists = os.path.exists(os.path.join(self._path, self.increment_name(name, 0)))

        # make sure that we dont overwrite if overwrite and increment are both false
        warning = "Filename and path already exists, refusing to set filename without overwrite=True or increment=True"
        assert not (exists and not overwrite and not increment), warning

        if overwrite is True:
            increment = False

        # otherwise overwrite the file
        if increment is False:
            self.filepath = os.path.join(self._path, self.increment_name(name, 0))
            self.name = self.increment_name(name, 0)
            return self.name

        # or increment the filename
        i = 0
        while os.path.exists(os.path.join(self._path, self.increment_name(name, i))):
            i += 1
        self.name = self.increment_name(name, i)
        self.filepath = os.path.join(self._path, self.increment_name(name, i))
        return self.name

    def set_local_path(self, directory, create_dir=False):
        """
            sets path for local saving of information
            if create is true we will create the folder even if it doesnt exist
        """
        if not os.path.exists(directory) and create_dir is True:
            os.makedirs(directory)

        if os.path.isdir(directory):
            self._path = directory

    def _prep_metadata(self):
        # copy the self.__dict__ and delete all that start with _
        obj = self.__dict__.copy()
        [obj.pop(item) for item in obj.keys() if item.startswith('_')]
        obj.pop('filepath')
        return obj

    def _process_response_metadata(self, response_metadata, **kw):
        self.__dict__.update(response_metadata["metadata"])

        self.id = response_metadata["id"]
        self.created_on = response_metadata["created_on"]
        self.updated_on = response_metadata["updated_on"]
        self._cloud_name = response_metadata["name"]
        self._rino_type = response_metadata["type"]

        if self.name is None:
            self.set_name(response_metadata["name"], **kw)

        # these are some variables we will keep hidden, marked with underscore
        self._size = response_metadata["size"]
        self._parent = response_metadata["parent"]

        return self

    def save_local_metadata(self):
        """
            save all the exposed variables to a json file
        """
        # save to the set local path and add .json
        with open(self.filepath + '.json', 'w+') as outfile:
            json.dump(self._prep_metadata(), outfile, indent=4)

    def import_local_metadata(self):
        with open(self.filepath + '.json', 'r') as infile:
            meta = json.loads(infile.read())
            self.__dict__.update(meta)

    def upload(self):
        meta = self._prep_metadata()
        meta["parent"] = self._parent

        r = rinocloud.http.upload(self.filepath, meta)
        assert r.status_code == 201, "Upload failed:\n%s" % r.text
        self._process_response_metadata(r.json())

    def upload_meta(self):
        meta = self._prep_metadata()
        meta["parent"] = self._parent
        r = rinocloud.http.upload_meta(meta)
        assert r.status_code == 201, "Upload failed:\n%s" % r.text
        self._process_response_metadata(r.json())

    def update(self):
        meta = self._prep_metadata()
        meta["parent"] = self._parent
        r = rinocloud.http.upload_meta(meta)
        assert r.status_code == 201, "Upload failed:\n%s" % r.text
        self._process_response_metadata(r.json())

    def get(self, id=None, truncate_metadata=True, **kw):
        _id = id
        if _id is None:
            if self.id is None:
                raise AttributeError("Can't fetch without an id. Need to either set Object.id or pass id to get.")
            else:
                _id = self.id

        r = rinocloud.http.get_metadata(_id, truncate_metadata=True)
        assert r.status_code != 404, "Object does not exist in Rinocloud. Error 404."
        self._process_response_metadata(r.json(), **kw)

    def download(self):
        assert self.id is not None, "Need to have id set to download data."
        assert self._rino_type == "file", "Target object is not a file, its a folder or empty object."

        r = rinocloud.http.download(self.id, self.filepath, self._size)
        assert r.status_code == 200, "Download error occured: %s" % r.text
        return self.filepath



# Rinocloud Python bindings
# API docs at http://github.com/rinocloud/rinocloud-python
# Authors:
# Jamie Lee <jamie@rinocloud.com>
# Eoin Murray <eoin@rinocloud.com>

api_key = None
api_domain = 'https://rinocloud.com/'
api_base = '%s/api/1/' % api_domain

path = ''

from config import *
from object import Object
from query import Query
from batch import Batch
import http as http

set_rinocloud_urls()




import rinocloud
import os


def set_local_path(directory, create_dir=False):
    """
        sets path for local saving of information
        if create is true we will create the folder even if it doesnt exist
    """
    if not os.path.exists(directory) and create_dir is True:
        os.makedirs(directory)

    if not os.path.exists(directory) and create_dir is False:
        raise AttributeError("Path '%s' does not exist, to make it pass create_dir=True to rinocloud.set_local_path" % directory)

    if os.path.isdir(directory):
        rinocloud.path = directory

    return directory


def set_domain(url):
    rinocloud.api_domain = url
    rinocloud.api_base = '%s/api/1/' % rinocloud.api_domain
    set_rinocloud_urls()


def set_rinocloud_urls():
    rinocloud.urls = {
        'upload': rinocloud.api_base + 'files/upload_multipart/',
        'upload_meta': rinocloud.api_base + 'files/create_object/',
        'get_metadata': rinocloud.api_base + 'files/get_metadata/',
        'update': rinocloud.api_base + 'files/update_metadata/',
        'download': rinocloud.api_base + 'files/download/?id=',
        'query': rinocloud.api_base + 'files/query/',
        'count': rinocloud.api_base + 'files/query_count/',
        'sign_s3': rinocloud.api_base + 'files/sign_s3/',
        'pre_s3_upload': rinocloud.api_base + 'files/pre_s3_upload/',
        'post_s3_upload': rinocloud.api_base + 'files/post_s3_upload/',
    }




import rinocloud
import requests
from requests_toolbelt import MultipartEncoder, MultipartEncoderMonitor
from clint.textui.progress import Bar as ProgressBar
from clint.textui import progress
import json


def upload(filepath=None, meta=None):
    encoder = MultipartEncoder(
        fields={
            'file': ('file', open(filepath, 'rb')),
            'json': json.dumps(meta)
        }
    )

    encoder_len = encoder.len
    bar = ProgressBar(expected_size=encoder_len, filled_char='#')

    def callback(monitor):
        bar.show(monitor.bytes_read)

    m = MultipartEncoderMonitor(encoder, callback)

    headers = {
        'Authorization': 'Token %s' % rinocloud.api_key,
        'X-Requested-With': 'XMLHttpRequest',
        'Content-Type': m.content_type
    }

    return requests.post(rinocloud.urls["upload"], data=m, headers=headers)


def upload_meta(meta):
    headers = {
        'Authorization': 'Token %s' % rinocloud.api_key,
        'X-Requested-With': 'XMLHttpRequest'
    }

    return requests.post(rinocloud.urls["upload_meta"], json=meta, headers=headers)


def get_metadata(_id, truncate_metadata=True):
    headers = {
        'Authorization': 'Token %s' % rinocloud.api_key,
        'X-Requested-With': 'XMLHttpRequest'
    }

    return requests.post(rinocloud.urls["get_metadata"], json={'id': _id, 'truncate_metadata': truncate_metadata}, headers=headers)


def download(_id, filepath, size):
    headers = {
        'Authorization': 'Token %s' % rinocloud.api_key,
        'X-Requested-With': 'XMLHttpRequest'
    }
    r = requests.get(rinocloud.urls["download"] + str(_id), stream=True, headers=headers)
    with open(filepath, 'wb') as f:
        total_length = size
        for chunk in progress.bar(r.iter_content(chunk_size=1024), expected_size=(total_length / 1024) + 1):
            if chunk:
                f.write(chunk)
                f.flush()
    return r


def query(query, truncate_metadata=True, limit=20, offset=0):
    headers = {
        'Authorization': 'Token %s' % rinocloud.api_key,
        'X-Requested-With': 'XMLHttpRequest'
    }

    payload = {
        'query': query,
        'truncate_metadata': truncate_metadata,
        'limit': limit,
        'offset': offset
    }

    return requests.post(rinocloud.urls["query"], json=payload, headers=headers)


def count(query):
    headers = {
        'Authorization': 'Token %s' % rinocloud.api_key,
        'X-Requested-With': 'XMLHttpRequest'
    }

    return requests.post(rinocloud.urls["count"], json={'query': query}, headers=headers)



from setuptools import setup
setup(
    name="rinocloud",
    version="0.0.2",
    packages=["rinocloud"],
    description='Rinocloud python bindings',
    author='Eoin Murray',
    author_email='eoin@rinocloud.com',
    url='https://github.com/rinocloud/rinocloud-python',  # use the URL to the github repo
    download_url='https://github.com/rinocloud/rinocloud-python/tarball/0.1',  # I'll explain this in a second
    keywords=['rinocloud', 'data', 'api'],  # arbitrary keywords
    classifiers=[],
    install_requires=[
        "requests",
        "requests_toolbelt",
        "arrow",
        "clint"
    ],
)
